/*
 * @brief FreeRTOS Display example
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2014
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "board.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "bios_loader.h"
#include "touch.h"
#include "display.h"
#include "spifi.h"

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

static const uint16_t COLORS[] = {
  0xffff, // WHITE      
  0xF800, // RED        
  0x07E0, // GREEN      
  0x7BEF, // LIGHTGRAY  
  0x001F, // BLUE       
  0xF81F, // MAGENTA    
  0x07FF, // CYAN       
  0x39E7, // DARKGRAY   
  0xFFE0, // YELLOW     
  0x7800, // LIGHTRED   
  0x03E0, // LIGHTGREEN 
  0x000F, // LIGHTBLUE  
  0x0000, // BLACK      
};
#define NUM_COLORS   (sizeof(COLORS)/sizeof(COLORS[0]))

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

static void crash()
{
  volatile int i;
  while(true) {
    Board_LED_Set(0, false);
    Board_LED_Set(1, false);
    Board_LED_Set(2, true);
    Board_LED_Set(3, true);
    
    for (i = 0; i < 1000000; i++) {
      __NOP();
    }
    
    Board_LED_Set(0, true);
    Board_LED_Set(1, true);
    Board_LED_Set(2, false);
    Board_LED_Set(3, false);
    
    for (i = 0; i < 1000000; i++) {
      __NOP();
    }
  }
}


/* Sets up system hardware */
static void prvSetupHardware(void)
{
    BiosLoaderError_t err;
    SpifiError_t serr;
    
    SystemCoreClockUpdate();
    Board_Init();
    
    /* Initialize the BIOS and display */
    err = bios_init();
    if (err == BiosLoaderError_Ok) {
      uint8_t type, maj, min, rev;
      char mac[6] = {0};
      bios_getMacAddress(mac);
      Board_ENET_SetMacADDR((uint8_t*)mac);
      DEBUGOUT("MAC: %02x:%02x:%02x:%02x:%02x:%02x\n", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
      bios_getStats(&type, &maj, &min, &rev);
      DEBUGOUT("BIOS: rev %d.%d.%d for BOARD%d\n", maj,min,rev, type);
      
      err = disp_init();
      if (err != BiosLoaderError_Ok) {
        DEBUGOUT("Failed to initialize the display, error %d\n", err);
        crash();
      }
    } else {
      DEBUGOUT("Failed to initialize BIOS, error %d\n", err);
      crash();
    }

    /* Initialize SPIFI */
    serr = spifi_init();
    if (serr != SpifiError_Ok) {
      DEBUGOUT("Failed to initialize SPIFI, error %d\n", serr);

      // It does not matter that the SPIFI could not be initialized as it is not
      // used by this particular example
    }

    /* Initial LED0 state is off */
    Board_LED_Set(0, false);
}

/* LED1 toggle thread */
static void vLEDTask1(void *pvParameters) {
    bool LedState = false;

    while (1) {
        Board_LED_Set(0, LedState);
        LedState = (bool) !LedState;

        /* About a 3Hz on/off toggle rate */
        vTaskDelay(configTICK_RATE_HZ / 6);
    }
}

/* LED2 toggle thread */
static void vLEDTask2(void *pvParameters) {
    bool LedState = false;

    while (1) {
        Board_LED_Set(1, LedState);
        LedState = (bool) !LedState;

        /* About a 7Hz on/off toggle rate */
        vTaskDelay(configTICK_RATE_HZ / 14);
    }
}

/* UART (or output) thread */
static void vUARTTask(void *pvParameters) {
    int tickCnt = 0;

    while (1) {
        DEBUGOUT("Tick: %d\r\n", tickCnt);
        tickCnt++;

        /* About a 1s delay here */
        vTaskDelay(configTICK_RATE_HZ);
    }
}

/* Cannot use memset as it is 8-bit and we need 16-bits per pixel */
static void fillScreen(uint16_t* fb, uint16_t color) {
  uint16_t w = disp_width();
  uint16_t h = disp_height();
  uint16_t x, y;
  for (y = 0; y < h; y++) {
    for (x = 0; x < w; x++) {
      *fb++ = color;
    }  
  }
}

/* GUI thread with rotating set of background colors */
static void vGUITaskColors(void *pvParameters) {
    BiosLoaderError_t err;
        
    /* No malloc(), so use fixed location of buffer */
    uint16_t* fb1 = (uint16_t*)SDRAM_ADDR_FB1;
    uint16_t* fb2 = (uint16_t*)SDRAM_ADDR_FB2;
    
    /* Clear (i.e. set to BLACK) */
    fillScreen(fb1, 0x0000);
    
    /* Turn on display */
    err = disp_powerUp(fb1, Res_16bit_rgb565, FrameRate_Normal);
    if (err != BiosLoaderError_Ok) {
      DEBUGOUT("Failed to power up the display, error %d\n", err);
      crash();
    }

    while (1) {
        int i;
        for (i = 0; i < NUM_COLORS; i++) {
          if (i & 1) {
            fillScreen(fb2, COLORS[i]);
            disp_setFramebuffer(fb2);
          } else {
            fillScreen(fb1, COLORS[i]);
            disp_setFramebuffer(fb1);
          }
          
          /* About a 1s delay here */
          vTaskDelay(configTICK_RATE_HZ);
        }
    }
}

static xTaskHandle touchSem = NULL;
static void onTouchEvent(void) {
    portBASE_TYPE xHigherPriorityTaskWoken;
    xSemaphoreGiveFromISR( touchSem, &xHigherPriorityTaskWoken );
    portYIELD_FROM_ISR( xHigherPriorityTaskWoken );
}

/* GUI thread with touch handling */
static void vGUITaskTouch(void *pvParameters) {
    BiosLoaderError_t err;
    touch_coordinate_t coord = {0};
    uint16_t w = disp_width();
    uint16_t h = disp_height(); 
        
    /* No malloc(), so use fixed location of buffer */
    uint16_t* fb1 = (uint16_t*)SDRAM_ADDR_FB1;
    
    /* Clear (i.e. set to WHITE) */
    fillScreen(fb1, 0xffff);
    
    /* Turn on display */
    err = disp_powerUp(fb1, Res_16bit_rgb565, FrameRate_Normal);
    if (err != BiosLoaderError_Ok) {
      DEBUGOUT("Failed to power up the display, error %d\n", err);
      crash();
    }
    
    /* Initialize the touch controller */
    err = touch_init();
    if (err != BiosLoaderError_Ok) {
      DEBUGOUT("Failed to initialize the touch controller, error %d\n", err);
      crash();
    }

    /* Create a semaphore so that the callback function can inform
       this thread that there is a new set of coordinates */
    vSemaphoreCreateBinary(touchSem);
    if (touchSem == NULL) {
      DEBUGSTR("Failed to create a semaphore for touch events\n");
      crash();
    }
    
    /* Register a callback function to receive touch events */
    touch_setListener(onTouchEvent);

#define MARGIN  (2)
    while (1) {
      if (xSemaphoreTake( touchSem, portMAX_DELAY ) == pdTRUE) {
        // new coordinates
        err = touch_read(&coord, 1);
        if (err == BiosLoaderError_Ok) {
          if (coord.z > 0) {
            if ((coord.x > MARGIN) && (coord.x < (w-MARGIN)) &&
                (coord.y > MARGIN) && (coord.y < (h-MARGIN))) {
              // ok, user touched at a location where we can draw a square
              int x, y;
              for (x = -MARGIN; x < MARGIN; x++) {
                for (y = -MARGIN; y < MARGIN; y++) {
                  fb1[(coord.y + y)*w + coord.x + x] = 0x0000; //BLACK
                }
              }
            }
          }
        }
      }
    }
}


/*****************************************************************************
 * Public functions
 ****************************************************************************/

void handleAssert(const char* file, int line)
{
  DEBUGOUT("Error in FreeRTOS on line %d of %s\n", line, file);
  crash();
}

/**
 * @brief    main routine for FreeRTOS display example
 * @return    Nothing, function should not exit
 */
int main(void)
{
    prvSetupHardware();

    /* LED1 toggle thread */
    xTaskCreate(vLEDTask1, (signed char *) "vTaskLed1",
                configMINIMAL_STACK_SIZE, NULL, (tskIDLE_PRIORITY + 1UL),
                (xTaskHandle *) NULL);

    /* LED2 toggle thread */
    xTaskCreate(vLEDTask2, (signed char *) "vTaskLed2",
                configMINIMAL_STACK_SIZE, NULL, (tskIDLE_PRIORITY + 1UL),
                (xTaskHandle *) NULL);

    /* UART output thread, simply counts seconds */
    xTaskCreate(vUARTTask, (signed char *) "vTaskUart",
                configMINIMAL_STACK_SIZE, NULL, (tskIDLE_PRIORITY + 1UL),
                (xTaskHandle *) NULL);

#if 0
    /* GUI thread with rotating set of background colors */
    xTaskCreate(vGUITaskColors, (signed char *) "vTaskGUIColors",
                configMINIMAL_STACK_SIZE, NULL, (tskIDLE_PRIORITY + 1UL),
                (xTaskHandle *) NULL);
#else
    /* GUI thread with touch handling */
    xTaskCreate(vGUITaskTouch, (signed char *) "vTaskGUITouch",
                configMINIMAL_STACK_SIZE, NULL, (tskIDLE_PRIORITY + 1UL),
                (xTaskHandle *) NULL);
#endif

    /* Start the scheduler */
    vTaskStartScheduler();

    /* Should never arrive here */
    return 1;
}

/**
 * @}
 */
