/*
 *  Copyright 2014 Embedded Artists AB
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

#include "board.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
//#include "timers.h"
#include "GUI.h"
#include "GUIDRV_Lin.h"
//#include "bios_loader.h"
#include "touch.h"
#include "display.h"
//#include "spifi.h"
#include <string.h>

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

static xTaskHandle emWinSem = NULL;
//static xTimerHandle emWinTimer = NULL;

//static uint32_t currentTimeMs = 0;

static uint32_t dispWidth = 0;
static uint32_t dispHeight = 0;
static uint32_t firstFB = 0;
static uint32_t fbSize = 0;
static uint32_t extraMem = 0;
static uint32_t extraMemSize = 0;
static int numFrameBuffers = 0;

static touch_coordinate_t coord = {0};

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

//static void msTickHandler( xTimerHandle pxTimer )
//{
//	(void)pxTimer;
//	currentTimeMs++;
//}

static void showFrameBuffer(int id)
{
	if (id >= 0 && id < numFrameBuffers) {
		disp_setFramebuffer((void*)(firstFB + id*fbSize));
	}
}

static void onTouchEvent(void) {
	GUI_PID_STATE ewCoord = {0};
	if (touch_read(&coord, 1) == BiosLoaderError_Ok) {
		ewCoord.x = coord.x;
		ewCoord.y = coord.y;
		ewCoord.Pressed = (coord.z == 0) ? 0 : 1;
		GUI_PID_StoreState(&ewCoord);
	}
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/**
 * @brief    main routine for FreeRTOS display example
 * @return    Nothing, function should not exit
 */
void emwinhal_init(int numFB)
{
	dispWidth = disp_width();
	dispHeight = disp_height();
	fbSize = disp_fbSize();
	numFrameBuffers = numFB;

    do {
		if (numFB < 1) {
            printf("Failed. Must have at least one frame buffer, wanted %d\n", numFB);
            break;
		}
		if ((numFB * fbSize) >= SDRAM_SZ_EMWIN) {
            printf("Failed. Too many frame buffers to fit in available memory, need 0x%x bytes have %x\n",
            		(numFB * fbSize), SDRAM_SZ_EMWIN);
            break;
		}

		firstFB = SDRAM_ADDR_EMWIN;
        memset((void*)firstFB, 0x0, fbSize * numFB);

        disp_setFramebuffer((void*)firstFB);

        extraMem = ((SDRAM_ADDR_EMWIN + fbSize * numFB) + 0x1ff) & 0xfffffc00;
        extraMemSize = SDRAM_ADDR_EMWIN + SDRAM_SZ_EMWIN - extraMem;
        if (extraMemSize == 0 || extraMemSize > SDRAM_SZ_EMWIN) {
            printf("Failed. Not enough reserved memory after allocating frame buffers\n");
            break;
        }
        memset((void*)extraMem, 0x0, extraMemSize);

        /* Register a callback function to receive touch events */
        touch_setListener(onTouchEvent);

//    	emWinTimer = xTimerCreate((const signed char * const)"Ms Ticker", 1, pdTRUE, 0, msTickHandler);

    } while(false);
}

/******************************************************************************
 * emWin HAL functions
 *****************************************************************************/

/*********************************************************************
*
*       GUI_X_Config
*
* Purpose:
*   Called during the initialization process in order to set up the
*   available memory for the GUI.
*/
void GUI_X_Config(void) {
  //
  // Assign memory to emWin
  //
  GUI_ALLOC_AssignMemory((void*)extraMem, extraMemSize);
  GUI_ALLOC_SetAvBlockSize(0x80);

//  GUI_SetSignalEventFunc(GUI_X_SignalEvent);
//  GUI_SetWaitEventFunc(GUI_X_WaitEvent);
//  GUI_SetWaitEventTimedFunc(GUI_X_WaitEventTimed);

  //
  // Set default font
  //
  GUI_SetDefaultFont(GUI_FONT_6X8);
}

/*********************************************************************
 *
 *       LCD_X_Config
 *
 * Purpose:
 *   Called during the initialization process in order to set up the
 *   display driver configuration.
 */
void LCD_X_Config(void) {

    //
    // Initialize MultiBuffering
    //
	if (numFrameBuffers > 1) {
		GUI_MULTIBUF_Config(numFrameBuffers);
	}


    GUI_DEVICE_CreateAndLink(&GUIDRV_Lin_16_API, GUICC_M565, 0, 0);

    //
    // Display driver configuration, required for Lin-driver
    //
    LCD_SetPosEx(0, 0, 0);
    if (LCD_GetSwapXYEx(0)) {
        LCD_SetSizeEx  (0, disp_height(), disp_width());
        LCD_SetVSizeEx (0, disp_height(), disp_width());
    } else {
        LCD_SetSizeEx  (0, disp_width(), disp_height());
        LCD_SetVSizeEx (0, disp_width(), disp_height());
    }
    LCD_SetVRAMAddrEx(0, (void*)0);

    // TODO: Add Custom callback routine (DMA) for copying the buffers
    //       Explained in the emWin User Manual rev 5.26 section 21.4.1
}

/*********************************************************************
 *
 *       LCD_X_DisplayDriver
 *
 * Purpose:
 *   This function is called by the display driver for several purposes.
 *   To support the according task the routine needs to be adapted to
 *   the display controller. Please note that the commands marked with
 *   'optional' are not cogently required and should only be adapted if
 *   the display controller supports these features.
 *
 * Parameter:
 *   LayerIndex - Index of layer to be configured
 *   Cmd        - Please refer to the details in the switch statement below
 *   pData      - Pointer to a LCD_X_DATA structure
 *
 * Return Value:
 *   < -1 - Error
 *     -1 - Command not handled
 *      0 - Ok
 */
int LCD_X_DisplayDriver(unsigned LayerIndex, unsigned Cmd, void * pData) {
    //	LCD_X_SETORG_INFO * pSetOrg;
    int r;
    U32 TouchOrientation;

    switch (Cmd) {
    //
    // Required
    //
    case LCD_X_INITCONTROLLER:
        //
        // Called during the initialization process in order to set up the
        // display controller and put it into operation. If the display
        // controller is not initialized by any external routine this needs
        // to be adapted by the customer...
        //
        // ...

        //
        // Set display size and video-RAM address
        //
        //		LCD_SetSizeEx (800, 480, 0);
        //		LCD_SetVSizeEx(800, 480, 0);
        LCD_SetVRAMAddrEx(0, (void*)firstFB);

        TouchOrientation = (GUI_MIRROR_X * LCD_GetMirrorXEx(0)) |
							(GUI_MIRROR_Y * LCD_GetMirrorYEx(0)) |
							(GUI_SWAP_XY  * LCD_GetSwapXYEx (0)) ;
        GUI_TOUCH_SetOrientation(TouchOrientation);

        return 0;
    case LCD_X_SETORG:
        //
        // Required for setting the display origin which is passed in the 'xPos' and 'yPos' element of p
        //

        // pSetOrg = (LCD_X_SETORG_INFO *)pData;
        //        LPC_LCD->UPBASE = VRAM_ADDR_PHYS +
        //                (pSetOrg->yPos * YSIZE_PHYS * /*PIXEL_WIDTH*/ 2);  // Needs to be set, before LCDC is enabled

        return 0;
    case LCD_X_SHOWBUFFER:
        {
            LCD_X_SHOWBUFFER_INFO * p = (LCD_X_SHOWBUFFER_INFO *)pData;
            //
            // Calculate address of the given buffer
            //
            //unsigned long BufferSize = (XSIZE * YSIZE * BITSPERPIXEL) / 8;
            //unsigned long Addr = _VRamBaseAddr + ewGui->getFrameBufferSize() * pData->Index;
            //
            // Make the given buffer visible
            //
            showFrameBuffer(p->Index);
            //
            // Send a confirmation that the buffer is visible now
            //
            GUI_MULTIBUF_Confirm(p->Index);
        }
        return 0;
    default:
        r = -1;
    }

    return r;
}

/*********************************************************************
 *
 *      Timing:
 *                 GUI_X_GetTime()
 *                 GUI_X_Delay(int)
 *
 * Some timing dependent routines require a GetTime
 * and delay function. Default time unit (tick), normally is 1 ms.
 */

int GUI_X_GetTime(void) {
	//return currentTimeMs;
	int ticks = xTaskGetTickCount();
	return ticks;
	//return xTaskGetTickCount();
}

void GUI_X_Delay(int ms) {
	portTickType delay = (configTICK_RATE_HZ * ms)/1000;
	vTaskDelay(delay);
	//vTaskDelay((configTICK_RATE_HZ * ms)/1000);
}

/*********************************************************************
*
*       GUI_X_Init()
*
* Note:
*     GUI_X_Init() is called from GUI_Init is a possibility to init
*     some hardware which needs to be up and running before the GUI.
*     If not required, leave this routine blank.
*/

void GUI_X_Init(void) {
	//xTimerStart(emWinTimer, 0);
}


/*********************************************************************
*
*       GUI_X_ExecIdle
*
* Note:
*  Called if WM is in idle state
*/

void GUI_X_ExecIdle(void) {}

/*********************************************************************
*
*      Multitasking:
*
*                 GUI_X_InitOS()
*                 GUI_X_GetTaskId()
*                 GUI_X_Lock()
*                 GUI_X_Unlock()
*
* Note:
*   The following routines are required only if emWin is used in a
*   true multi task environment, which means you have more than one
*   thread using the emWin API.
*   In this case the
*                       #define GUI_OS 1
*  needs to be in GUIConf.h
*/


//static OS_RSEMA RSema;

void GUI_X_InitOS(void) {
	vSemaphoreCreateBinary(emWinSem);
}
void GUI_X_Unlock(void) {
	xSemaphoreGive( emWinSem );
}
void GUI_X_Lock(void) {
	xSemaphoreTake( emWinSem, portMAX_DELAY );
}
U32 GUI_X_GetTaskId(void) {
	return (U32)xTaskGetCurrentTaskHandle();
}

/*********************************************************************
*
*      Event driving (optional with multitasking)
*
*                 GUI_X_WaitEvent()
*                 GUI_X_WaitEventTimed()
*                 GUI_X_SignalEvent()
*/

//static void emTimeout(void const* args) {
//	GUI_X_SignalEvent();
//}
//
//void GUI_X_WaitEvent(void) {
//    Thread::signal_wait(EMWIN_TASK_SIGNAL); //wait forever
//}
//
//void GUI_X_SignalEvent(void) {
//	if (_emTask) {
//		osSignalSet(_emTask, EMWIN_TASK_SIGNAL);
//	}
//}
//
//void GUI_X_WaitEventTimed(int Period) {
//    static RtosTimer _emTim(emTimeout, osTimerOnce);
//    if (Period > 0) {
//
//        //wait no more than Period milliseconds
//        _emTim.start(Period);
//        Thread::signal_wait(EMWIN_TASK_SIGNAL);
//        _emTim.stop();
//    }
//}

/*********************************************************************
 *
 *      Logging: OS dependent

Note:
  Logging is used in higher debug levels only. The typical target
  build does not use logging and does therefor not require any of
  the logging routines below. For a release build without logging
  the routines below may be eliminated to save some space.
  (If the linker is not function aware and eliminates unreferenced
  functions automatically)

 */

void GUI_X_Log     (const char *s) { printf("emWin[Log] %s\n", s); }
void GUI_X_Warn    (const char *s) { printf("emWin[Log] %s\n", s); }
void GUI_X_ErrorOut(const char *s) { printf("emWin[Log] %s\n", s); }

/*********************************************************************
 *
 *       GUI_TOUCH_X_ActivateX()
 *
 * Function decription:
 *   Called from GUI, if touch support is enabled.
 *   Switches on voltage on X-axis,
 *   prepares measurement for Y-axis.
 *   Voltage on Y-axis is switched off.
 */
void GUI_TOUCH_X_ActivateX(void) {
}

/*********************************************************************
 *
 *       GUI_TOUCH_X_ActivateY()
 *
 * Function decription:
 *   Called from GUI, if touch support is enabled.
 *   Switches on voltage on Y-axis,
 *   prepares measurement for X-axis.
 *   Voltage on X-axis is switched off.
 */
void GUI_TOUCH_X_ActivateY(void) {
}

/*********************************************************************
 *
 *       GUI_TOUCH_X_MeasureX()
 *
 * Function decription:
 *   Called from GUI, if touch support is enabled.
 *   Measures voltage of X-axis.
 */
int  GUI_TOUCH_X_MeasureX(void) {
    return coord.x;
}

/*********************************************************************
 *
 *       GUI_TOUCH_X_MeasureY()
 *
 * Function decription:
 *   Called from GUI, if touch support is enabled.
 *   Measures voltage of Y-axis.
 */
int  GUI_TOUCH_X_MeasureY(void) {
    return coord.y;
}
