/*
 * @brief FreeRTOS Display example
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2014
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "board.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "bios_loader.h"
#include "touch.h"
#include "display.h"
#include "spifi.h"
#include "emWinHal.h"
#include "GUI.h"

#include "lwip/init.h"
#include "lwip/opt.h"
#include "lwip/sys.h"
#include "lwip/memp.h"
#include "lwip/tcpip.h"
#include "lwip/ip_addr.h"
#include "lwip/netif.h"
#include "lwip/timers.h"
#include "netif/etharp.h"

#if LWIP_DHCP
#include "lwip/dhcp.h"
#endif

#include "arch\lpc17xx_40xx_emac.h"
#include "arch\lpc_arch.h"
#include "arch\sys_arch.h"
#include "lpc_phy.h"/* For the PHY monitor support */
#include "tcpecho.h"

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

/* NETIF data */
static struct netif lpc_netif;

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

static void crash()
{
  volatile int i;
  while(true) {
    Board_LED_Set(0, false);
    Board_LED_Set(1, false);
    Board_LED_Set(2, true);
    Board_LED_Set(3, true);
    
    for (i = 0; i < 1000000; i++) {
      __NOP();
    }
    
    Board_LED_Set(0, true);
    Board_LED_Set(1, true);
    Board_LED_Set(2, false);
    Board_LED_Set(3, false);
    
    for (i = 0; i < 1000000; i++) {
      __NOP();
    }
  }
}


/* Sets up system hardware */
static void prvSetupHardware(void)
{
    BiosLoaderError_t err;
    SpifiError_t serr;
    
    SystemCoreClockUpdate();
    Board_Init();
    
    /* Initialize the BIOS and display */
    err = bios_init();
    if (err == BiosLoaderError_Ok) {
      uint8_t type, maj, min, rev;
      char mac[6] = {0};
      bios_getMacAddress(mac);
      Board_ENET_SetMacADDR((uint8_t*)mac);
      DEBUGOUT("MAC: %02x:%02x:%02x:%02x:%02x:%02x\n", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
      bios_getStats(&type, &maj, &min, &rev);
      DEBUGOUT("BIOS: rev %d.%d.%d for BOARD%d\n", maj,min,rev, type);
      
      err = disp_init();
      if (err != BiosLoaderError_Ok) {
        DEBUGOUT("Failed to initialize the display, error %d\n", err);
        crash();
      }
    } else {
      DEBUGOUT("Failed to initialize BIOS, error %d\n", err);
      crash();
    }

    /* Initialize SPIFI */
    serr = spifi_init();
    if (serr != SpifiError_Ok) {
      DEBUGOUT("Failed to initialize SPIFI, error %d\n", serr);

      // It does not matter that the SPIFI could not be initialized as it is not
      // used by this particular example
    }

    /* Initial LED0 state is off */
    Board_LED_Set(0, false);
}

/* LED1 toggle thread */
static void vLEDTask1(void *pvParameters) {
    bool LedState = false;

    while (1) {
        Board_LED_Set(0, LedState);
        LedState = (bool) !LedState;

        /* About a 3Hz on/off toggle rate */
        vTaskDelay(configTICK_RATE_HZ / 6);
    }
}

/* LED2 toggle thread */
static void vLEDTask2(void *pvParameters) {
    bool LedState = false;

    while (1) {
        Board_LED_Set(1, LedState);
        LedState = (bool) !LedState;

        /* About a 7Hz on/off toggle rate */
        vTaskDelay(configTICK_RATE_HZ / 14);
    }
}

/* UART (or output) thread */
static void vUARTTask(void *pvParameters) {
    int tickCnt = 0;

    while (1) {
        DEBUGOUT("Tick: %d\r\n", tickCnt);
        tickCnt++;

        /* About a 1s delay here */
        vTaskDelay(configTICK_RATE_HZ);
    }
}

/* Cannot use memset as it is 8-bit and we need 16-bits per pixel */
static void fillScreen(uint16_t* fb, uint16_t color) {
  uint16_t w = disp_width();
  uint16_t h = disp_height();
  uint16_t x, y;
  for (y = 0; y < h; y++) {
    for (x = 0; x < w; x++) {
      *fb++ = color;
    }  
  }
}


extern void MainTask(void);

/* GUI thread running emWin */
static void vGUITaskEmWin(void *pvParameters) {
    BiosLoaderError_t err;

    /* No malloc(), so use fixed location of buffer */
    uint16_t* fb1 = (uint16_t*)SDRAM_ADDR_EMWIN;

    /* Clear (i.e. set to WHITE) */
    fillScreen(fb1, 0xffff);

    /* Turn on display */
    err = disp_powerUp(fb1, Res_16bit_rgb565, FrameRate_Normal);
    if (err != BiosLoaderError_Ok) {
      DEBUGOUT("Failed to power up the display, error %d\n", err);
      crash();
    }

    /* Initialize the touch controller */
    err = touch_init();
    if (err != BiosLoaderError_Ok) {
      DEBUGOUT("Failed to initialize the touch controller, error %d\n", err);
      crash();
    }

    emwinhal_init(3);//, 12*1024*1024);

	/* emWin start */
	GUI_Init();

	MainTask();
//	while (true) {
//		/* Solid color display */
//		GUI_SetBkColor(GUI_RED);
//		GUI_Clear();
//		GUI_Delay(1000);
//		GUI_SetBkColor(GUI_GREEN);
//		GUI_Clear();
//		GUI_Delay(1000);
//		GUI_SetBkColor(GUI_BLUE);
//		GUI_Clear();
//		GUI_Delay(1000);
//		GUI_SetBkColor(GUI_BLACK);
//		GUI_Clear();
//	}
}

/* Callback for TCPIP thread to indicate TCPIP init is done */
static void tcpip_init_done_signal(void *arg)
{
	/* Tell main thread TCP/IP init is done */
	*(s32_t *) arg = 1;
}

/* LWIP kickoff and PHY link monitor thread */
static void vSetupIFTask (void *pvParameters) {
	ip_addr_t ipaddr, netmask, gw;
	volatile s32_t tcpipdone = 0;
	uint32_t physts;
	static int prt_ip = 0;

	/* Wait until the TCP/IP thread is finished before
	   continuing or wierd things may happen */
	LWIP_DEBUGF(LWIP_DBG_ON, ("Waiting for TCPIP thread to initialize...\n"));
	tcpip_init(tcpip_init_done_signal, (void *) &tcpipdone);
	while (!tcpipdone) {
		msDelay(1);
	}

	LWIP_DEBUGF(LWIP_DBG_ON, ("Starting LWIP TCP echo server...\n"));

	/* Static IP assignment */
#if LWIP_DHCP
	IP4_ADDR(&gw, 0, 0, 0, 0);
	IP4_ADDR(&ipaddr, 0, 0, 0, 0);
	IP4_ADDR(&netmask, 0, 0, 0, 0);
#else
	IP4_ADDR(&gw, 10, 1, 10, 1);
	IP4_ADDR(&ipaddr, 10, 1, 10, 234);
	IP4_ADDR(&netmask, 255, 255, 255, 0);
#endif

	/* Add netif interface for lpc17xx_8x */
	if (!netif_add(&lpc_netif, &ipaddr, &netmask, &gw, NULL, lpc_enetif_init,
				   tcpip_input)) {
		LWIP_ASSERT("Net interface failed to initialize\r\n", 0);
	}
	netif_set_default(&lpc_netif);
	netif_set_up(&lpc_netif);

	/* Enable MAC interrupts only after LWIP is ready */
	NVIC_SetPriority(ETHERNET_IRQn, config_ETHERNET_INTERRUPT_PRIORITY);
	NVIC_EnableIRQ(ETHERNET_IRQn);

#if LWIP_DHCP
	dhcp_start(&lpc_netif);
#endif

	/* Initialize and start application */
	tcpecho_init();

	/* This loop monitors the PHY link and will handle cable events
	   via the PHY driver. */
	while (1) {
		/* Call the PHY status update state machine once in a while
		   to keep the link status up-to-date */
		physts = lpcPHYStsPoll();

		/* Only check for connection state when the PHY status has changed */
		if (physts & PHY_LINK_CHANGED) {
			if (physts & PHY_LINK_CONNECTED) {
				Board_LED_Set(0, true);
				prt_ip = 0;

				/* Set interface speed and duplex */
				if (physts & PHY_LINK_SPEED100) {
					Chip_ENET_Set100Mbps(LPC_ETHERNET);
					NETIF_INIT_SNMP(&lpc_netif, snmp_ifType_ethernet_csmacd, 100000000);
				}
				else {
					Chip_ENET_Set10Mbps(LPC_ETHERNET);
					NETIF_INIT_SNMP(&lpc_netif, snmp_ifType_ethernet_csmacd, 10000000);
				}
				if (physts & PHY_LINK_FULLDUPLX) {
					Chip_ENET_SetFullDuplex(LPC_ETHERNET);
				}
				else {
					Chip_ENET_SetHalfDuplex(LPC_ETHERNET);
				}

				tcpip_callback_with_block((tcpip_callback_fn) netif_set_link_up,
										  (void *) &lpc_netif, 1);
			}
			else {
				Board_LED_Set(0, false);
				tcpip_callback_with_block((tcpip_callback_fn) netif_set_link_down,
										  (void *) &lpc_netif, 1);
			}

			DEBUGOUT("Link connect status: %d\r\n", ((physts & PHY_LINK_CONNECTED) != 0));

			/* Delay for link detection (250mS) */
			vTaskDelay(configTICK_RATE_HZ / 4);
		}

		/* Print IP address info */
		if (!prt_ip) {
			if (lpc_netif.ip_addr.addr) {
				static char tmp_buff[16];
				DEBUGOUT("IP_ADDR    : %s\r\n", ipaddr_ntoa_r((const ip_addr_t *) &lpc_netif.ip_addr, tmp_buff, 16));
				DEBUGOUT("NET_MASK   : %s\r\n", ipaddr_ntoa_r((const ip_addr_t *) &lpc_netif.netmask, tmp_buff, 16));
				DEBUGOUT("GATEWAY_IP : %s\r\n", ipaddr_ntoa_r((const ip_addr_t *) &lpc_netif.gw, tmp_buff, 16));
				prt_ip = 1;
			}
		}
	}
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

void handleAssert(const char* file, int line)
{
  DEBUGOUT("Error in FreeRTOS on line %d of %s\n", line, file);
  crash();
}

void HardFault_Handler(void)
{
	DEBUGOUT("Intercepted HardFault\n");
	crash();
}

/**
 * @brief	MilliSecond delay function based on FreeRTOS
 * @param	ms	: Number of milliSeconds to delay
 * @return	Nothing
 * Needed for some functions, do not use prior to FreeRTOS running
 */
void msDelay(uint32_t ms)
{
	vTaskDelay((configTICK_RATE_HZ * ms) / 1000);
}


/**
 * @brief    main routine for FreeRTOS display example
 * @return    Nothing, function should not exit
 */
int main(void)
{
    prvSetupHardware();

#if 0
    /* LED1 toggle thread */
    xTaskCreate(vLEDTask1, (signed char *) "vTaskLed1",
                configMINIMAL_STACK_SIZE, NULL, (tskIDLE_PRIORITY + 1UL),
                (xTaskHandle *) NULL);

    /* LED2 toggle thread */
    xTaskCreate(vLEDTask2, (signed char *) "vTaskLed2",
                configMINIMAL_STACK_SIZE, NULL, (tskIDLE_PRIORITY + 1UL),
                (xTaskHandle *) NULL);

    /* UART output thread, simply counts seconds */
    xTaskCreate(vUARTTask, (signed char *) "vTaskUart",
                configMINIMAL_STACK_SIZE, NULL, (tskIDLE_PRIORITY + 1UL),
                (xTaskHandle *) NULL);
#endif

	/* Add another thread for initializing physical interface. This
	   is delayed from the main LWIP initialization. */
	xTaskCreate(vSetupIFTask, (signed char *) "SetupIFx",
				configMINIMAL_STACK_SIZE*2, NULL, (tskIDLE_PRIORITY + 1UL),
				(xTaskHandle *) NULL);

    /* GUI thread with emWin */
    xTaskCreate(vGUITaskEmWin, (signed char *) "vTaskGUIemWin",
                configMINIMAL_STACK_SIZE*8, NULL, (tskIDLE_PRIORITY + 1UL),
                (xTaskHandle *) NULL);

    /* Start the scheduler */
    vTaskStartScheduler();

    /* Should never arrive here */
    return 1;
}

/**
 * @}
 */
